@@ -6,11 +6,16 @@ import warnings
 
 class DataNode(BaseNode):
     
-    def __init__(self, node_uid, value=None, persist=False, verbose=False, alias=None):
-        super(DataNode, self).__init__(node_uid, 'data', verbose, alias or 'data')
+    def __init__(self, graph_uid, graph_alias, node_uid, value=None, persist=False, verbose=False, alias=None, graph_dict=None):
+        super(DataNode, self).__init__(graph_uid, graph_alias, node_uid, 'data', verbose, alias or 'data')
         
-        self.value_holder = DataHolderNode(self.node_uid, value, self.verbose)
+        self.value_holder = DataHolderNode(graph_uid, graph_alias, self.node_uid, value, self.verbose)
+
         self.data_persist = persist
+        self.graph_uid = graph_uid
+        self.graph_alias = graph_alias
+
+        self.graph_dict = graph_dict
 
     def set_value(self, value):
         
@@ -29,8 +34,24 @@ class DataNode(BaseNode):
         return self.data_persist
 
     def get(self):
+
+        # this is to support the multi-graph paradigm
+        self.remove_dead_child_nodes()
         
         if self.value_holder.has_value():
+
+            # update graph_dict 
+            # during a computation execution, the value_holder can hold transient data
+            # so we need to check that this data node is indeed persisted
+            # as well as actually has data
+            if self.is_persisted():
+                if self.verbose:
+                    print('persisted', self.node_uid)
+
+                data_dim = self.value_holder.get_data_dim_as_str()
+
+                self.graph_dict[self.node_uid]['data_dim'] = data_dim
+
             return self.value_holder.get()
         else:
             if self.verbose:
@@ -38,6 +59,16 @@ class DataNode(BaseNode):
             
             self.activate_dependency_op_nodes()
             self.parent_node_weak_refs[0]().run()
+
+            # update graph_dict
+            if self.is_persisted():
+                if self.verbose:
+                    print('persisted', self.node_uid)
+
+                data_dim = self.value_holder.get_data_dim_as_str()
+
+                self.graph_dict[self.node_uid]['data_dim'] = data_dim
+
             return self.value_holder.get()
 
     def activate_dependency_op_nodes(self):
@@ -83,6 +114,6 @@ class DataNode(BaseNode):
             warnings.warn("You are releasing a DataNode that was persisted!", RuntimeWarning)
         
         del self.value_holder
-        self.value_holder = DataHolderNode(self.node_uid, None, self.verbose)
+        self.value_holder = DataHolderNode(self.graph_uid, self.graph_alias, self.node_uid, None, self.verbose)
         
         
\ No newline at end of file
